/*
 * fragment.h
 *
 * Copyright 2022 Jonathan Schöbel <jonathan@Ubermos-2019>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 *
 *
 */


#ifndef __FRAGMENT_H__
#define __FRAGMENT_H__

#if !defined (__SEFHT_H_INSIDE__) && !defined (LIB_SEFHT_COMPILATION)
#error "Only <sefht/sefht.h> can be included directly."
#endif

#include "status.h"

#include "data.h"
#include "text.h"


enum HTML_MODE
{
	INLINE,
	WRAP
};

#ifdef LIB_SEFHT_COMPILATION

enum SH_FRAGMENT_TYPE
{
	NODE
};

struct fragment_methods;

struct SH_Fragment;
struct SH_Fragment
{
	struct SH_Data * data;

	enum SH_FRAGMENT_TYPE type;
	const struct fragment_methods * methods;
};

struct fragment_methods
{
	struct SH_Fragment * (*copy) (struct SH_Fragment *,
				      struct SH_Status *);

	void (*free) (struct SH_Fragment *);

	struct SH_Text * (*to_html) (struct SH_Fragment *,
				     enum HTML_MODE,
				     unsigned int,
				     unsigned int,
				     const char *,
				     struct SH_Status *);
};


static inline
void
init_fragment (struct SH_Fragment * fragment,
	       struct SH_Data * data,
	       const enum SH_FRAGMENT_TYPE type,
	       const struct fragment_methods * const methods)
{
	fragment->data = data;
	fragment->type = type;
	fragment->methods = methods;

	return;
}

static inline
void
free_fragment (struct SH_Fragment * fragment)
{
	return;
}

static inline
void
copy_fragment (struct SH_Fragment * copy,
	       struct SH_Fragment * fragment)
{
	copy->data = fragment->data;
	copy->type = fragment->type;
	copy->methods = fragment->methods;

	return;
}

static inline
enum SH_FRAGMENT_TYPE
get_type (struct SH_Fragment * fragment)
{
	return fragment->type;
}

#endif /* LIB_SEFHT_COMPILATION */

typedef struct SH_Fragment SH_Fragment;

SH_Fragment *
SH_Fragment_copy (SH_Fragment * fragment,
                  struct SH_Status * status);

void
SH_Fragment_free (SH_Fragment * fragment);

SH_Text *
SH_Fragment_to_html (SH_Fragment * fragment,
                     enum HTML_MODE mode,
                     unsigned int indent_base,
                     unsigned int indent_step,
                     char * indent_char,
                     struct SH_Status * status);

#endif /* __FRAGMENT_H__ */
