/*
 * node_fragment.h
 *
 * Copyright 2022 Jonathan Schöbel <jonathan@Ubermos-2019>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 *
 *
 */


#ifndef __NODE_FRAGMENT_H__
#define __NODE_FRAGMENT_H__

#if !defined (__SEFHT_H_INSIDE__) && !defined (LIB_SEFHT_COMPILATION)
#status "Only <sefht/sefht.h> can be included directly."
#endif

#include "status.h"

#include "data.h"
#include "text.h"

#include "fragment.h"


#ifdef LIB_SEFHT_COMPILATION

struct SH_NodeFragment;
struct SH_NodeFragment
{
	struct SH_Fragment base;

	char * tag;

	size_t child_n;
	struct SH_Fragment ** childs;
};

#define OPEN_TAG_BEGIN "<"
#define OPEN_TAG_END ">"
#define CLOSE_TAG_BEGIN "</"
#define CLOSE_TAG_END ">"

//~ #define INDENT_TEXT "\t"
#define NEWLINE "\n"

#endif /* LIB_SEFHT_COMPILATION */

/* TODO: no-export */
#define INDENT_TEXT "\t"

typedef struct SH_NodeFragment SH_NodeFragment;


SH_Fragment *
SH_NodeFragment_new (const char * tag,
                     SH_Data * data,
                     struct SH_Status * status);

void
SH_NodeFragment_free (SH_NodeFragment * fragment);

SH_Fragment *
SH_NodeFragment_copy (SH_NodeFragment * fragment,
                      struct SH_Status * status);

SH_Fragment *
SH_NodeFragment_deepcopy (SH_NodeFragment * fragment,
                          struct SH_Status * status);

bool
SH_Fragment_is_NodeFragment (SH_Fragment * fragment);

char *
SH_NodeFragment_get_tag (SH_NodeFragment * fragment,
                         struct SH_Status * status);

SH_Fragment *
SH_NodeFragment_get_child (SH_NodeFragment * fragment,
                           size_t index,
                           struct SH_Status * status);

bool
SH_NodeFragment_is_child (SH_NodeFragment * fragment,
                          SH_Fragment * child);

bool
SH_NodeFragment_is_descendant (SH_NodeFragment * fragment,
                               SH_Fragment * child);

bool
SH_NodeFragment_append_child (SH_NodeFragment * fragment,
			      SH_Fragment * child,
                              struct SH_Status * status);

SH_Text *
SH_NodeFragment_to_html (SH_NodeFragment * fragment,
                         enum HTML_MODE mode,
                         unsigned int indent_base,
                         unsigned int indent_step,
                         char * indent_char,
                         struct SH_Status * status);

#endif /* __NODE_FRAGMENT_H__ */
