/*
 * text.h
 *
 * Copyright 2022 Jonathan Schöbel <jonathan@Ubermos-2019>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 *
 *
 */


#ifndef __TEXT_H__
#define __TEXT_H__

#if !defined (__SEFHT_H_INSIDE__) && !defined (LIB_SEFHT_COMPILATION)
#error "Only <sefht/sefht.h> can be included directly."
#endif

#include <stdbool.h>

#include "status.h"


#ifdef LIB_SEFHT_COMPILATION

#ifndef CHUNK_SIZE
#define CHUNK_SIZE 64
#endif /* CHUNK_SIZE */

#if CHUNK_SIZE == 0
#error "CHUNK_SIZE can't be 0."
#endif /* CHUNK_SIZE == 0 */

/*@abstract@*/
struct text_segment;
struct text_segment
{
	size_t length;
	size_t size;
	char * text;
	/*@relnull@*/ struct text_segment * next;
};

/*@abstract@*/
struct SH_Text
{
	/*@notnull@*/ struct text_segment * data;
};

#endif /* LIB_SEFHT_COMPILATION */

typedef /*@abstract@*/ struct SH_Text SH_Text;


/*@null@*/
/*@only@*/
struct SH_Text *
SH_Text_new (/*@null@*/ /*@out@*/ struct SH_Status * status);

/*@null@*/
/*@only@*/
struct SH_Text *
SH_Text_new_from_string (const char * string,
                         /*@null@*/ /*@out@*/ struct SH_Status * status);

void
SH_Text_free (/*@only@*/ struct SH_Text * text);

/*@null@*/
/*@only@*/
struct SH_Text *
SH_Text_copy (const struct SH_Text * text,
              /*@null@*/ /*@out@*/ struct SH_Status * status);

size_t
SH_Text_get_length (const struct SH_Text * text,
                    /*@null@*/ /*@out@*/ struct SH_Status * status);

/*@null@*/
/*@only@*/
char *
SH_Text_get_char (const struct SH_Text * text,
                  size_t index,
                  /*@null@*/ /*@out@*/ struct SH_Status * status);

/*@null@*/
/*@only@*/
char *
SH_Text_get_string (const struct SH_Text * text,
                    size_t index, size_t offset,
                    /*@null@*/ /*@out@*/ size_t * length,
                    /*@null@*/ /*@out@*/ struct SH_Status * status);

/*@null@*/
/*@only@*/
char *
SH_Text_get_range (const struct SH_Text * text,
                   size_t start, size_t end,
                   /*@null@*/ /*@out@*/ size_t * length,
                   /*@null@*/ /*@out@*/ struct SH_Status * status);

bool
SH_Text_append_string (struct SH_Text * text,
                       const char * string,
                       /*@null@*/ /*@out@*/ struct SH_Status * status);

bool
SH_Text_append_text (struct SH_Text * text,
                     const struct SH_Text * text2,
                     /*@null@*/ /*@out@*/ struct SH_Status * status);

void
SH_Text_join (struct SH_Text * text,
              /*@only@*/ struct SH_Text * text2);

void
SH_Text_print (const struct SH_Text * text);

#endif /* __TEXT_H__ */
