/*
 * validator_tag.h
 *
 * Copyright 2022 Jonathan Schöbel <jonathan@Ubermos-2019>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 *
 *
 */


#ifndef __VALIDATOR_TAG_H__
#define __VALIDATOR_TAG_H__

#if !defined (__VALIDATOR_H__)
#error "Please include only <sefht/validator.h>."
#endif

#include <stdbool.h>
#include <stdint.h>

#include "status.h"


typedef unsigned int Tag;
#define TAG_ERR (Tag) 0
#define TAG_MIN (Tag) 1
#define TAG_MAX (Tag) SIZE_MAX

#ifdef LIB_SEFHT_COMPILATION

#define NEXT_TAG(tag) (tag + 1)

struct tag_info
{
	union
	{
		struct
		{
			Tag id;
			/*@only@*/ char * name;
		} data;
		size_t next;
	};
};

#define TAG_DATA                                                       \
	/*@only@*/ struct tag_info * tags;                             \
	size_t tag_n;                                                  \
	Tag last_tag;                                                  \


#endif /* LIB_SEFHT_COMPILATION */



Tag
SH_Validator_register_tag (SH_Validator * validator,
                           const char * tag,
                           /*@null@*/ /*@out@*/
                           struct SH_Status * status);

bool
SH_Validator_deregister_tag (SH_Validator * validator,
                             Tag id,
                             /*@null@*/ /*@out@*/
                             struct SH_Status * status);

bool
SH_Validator_check_tag (SH_Validator * validator,
                        const char * tag);

#endif /* __VALIDATOR_TAG_H__ */
