/*
 * main.c
 *
 * Copyright 2022 Jonathan Schöbel <jonathan@Ubermos-2019>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 *
 *
 */


#include <stdio.h>
#include <stdlib.h>

#include <sefht/sefht.h>

int main(int argc, char **argv)
{
	page_t page;

	/* startup */
	SH_Cms * cms = SH_Cms_new (NULL);
	/* startup */

	page = SH_Cms_register_page (cms, "Startpage", NULL);

	/* shutdown */
	SH_Cms_free (cms);
	/* shutdown */


	SH_Data * data;
	SH_Fragment * root;
	SH_Fragment * child1;
	SH_Fragment * child2;
	SH_Text * text;

	data = SH_Data_new (NULL);

	root = SH_NodeFragment_new ("html", data, NULL);

	child1 = SH_NodeFragment_new ("head", data, NULL);
	SH_NodeFragment_append_child ((struct SH_NodeFragment *) root,
				      child1, NULL);

	child2 = SH_NodeFragment_new ("title", data, NULL);
	SH_NodeFragment_append_child ((struct SH_NodeFragment *) child1,
				      child2, NULL);


	child1 = SH_NodeFragment_new ("body", data, NULL);
	SH_NodeFragment_append_child ((struct SH_NodeFragment *) root,
				      child1, NULL);

	child2 = SH_NodeFragment_new ("header", data, NULL);
	SH_NodeFragment_append_child ((struct SH_NodeFragment *) child1,
				      child2, NULL);

	child2 = SH_NodeFragment_new ("main", data, NULL);
	SH_NodeFragment_append_child ((struct SH_NodeFragment *) child1,
				      child2, NULL);

	child2 = SH_NodeFragment_new ("footer", data, NULL);
	SH_NodeFragment_append_child ((struct SH_NodeFragment *) child1,
				      child2, NULL);

	text = SH_Fragment_to_html (root, WRAP, 0, 1, INDENT_TEXT, NULL);
	SH_Text_print (text);
	printf ("\n");

	SH_Text_free (text);

	text = SH_Fragment_to_html (root, INLINE, 0, 1, INDENT_TEXT, NULL);
	SH_Text_print (text);
	printf ("\n");

	SH_Text_free (text);

	SH_Fragment_free (root);

	SH_Data_free (data);

	return 0;
}

